;==========================================================
;+ Gestion de l'alimentation d'un appareil {fer  souder} +
;+ Prvue pour une dure de 20mn env.			  +
;+ Pour changer la dure, modifier b1 et b2		  +
;+							  +
;+ Ecrit par Aurelien ROBERT, 04/2000			  +
;+ http://www.aurelienr.com * aurelienr@hotmail.com	  +
;+							  +
;+ Avec un PIC 12C508A					  +
;+ OSC = RC Int / WDT = Off / MCLR = Off (disable) 	  +
;==========================================================


; au demarrage le PIC lance la temporisation
; il lance 250 fois une boucle (b1=250) repetant 50 fois (b2=50) un timer de 100ms
; si appui sur le poussoir, remise  zero du compteur : verif ttes les 100ms
; si boucle principale (b1 decrement) >100, LED verte allume (gpio 1)
;					= 100 (et <100), LED orange allume (gpio 2)
;					= 50 (et <50), LED rouge allume (gpio 4)
;					= 1, buzzer pendant env. 1 sec
; fin decompte et decollage relais, puis attente d'un appui poussoir
; sortie vers relais  gpio 0
; gpio5 vers buzzer
; entre  gpio3 (entre force par le C)

; variables, registres
; ====================
	cblock	0x0c 
		var1, var2, sorties, endtps, b1, b2
	endc

gpio	equ	0x06	; registre E/S
status	equ	0x03	; registre status

	org	0
	goto	start

	org	10

; initialisation
; ==============
start
	clrf	gpio
	movlw	B'00001000'	; gp3 entree, le reste en sorties
	tris	gpio
	movlw	0
	movwf	endtps
	movlw	D'250'
	movwf	b1
	movlw	B'00000011'
	movwf	sorties
	b	main

; arrt decompte
; ==============
stop
	movlw	B'00010110'	; les 3 LEDs allumes
	movwf	gpio
	btfsc	gpio,3		; attente de l'appui sur le poussoir
	b	$-1
	b	main


; routines de temporisation
; =========================
tempo					; 330 s
	movlw	D'107'	; 1s
	movwf	var1	; 1s
	decf	var1,f	; 1s*107
	call	loop	; 2s*107
	movf	var1,f	; 1s
	btfss	status,2; 1s
	b	$-4	; 2s
	return		; 1s venant de btfss + 2s
loop					; 305 s		; total = 100650s = 100.650ms
	movlw	D'100'	; 1s
	movwf	var2	; 1s
	decfsz	var2,f	; 1s*100
	b	$-1	; 2s*100
	return		; 1s venant de decfsz + 2s

; routine d'valuation du temps ecoul
; ====================================
testtps
	movlw	D'100'
	subwf	b1,0		; soustraction b1 - 100
	btfss	status,2	; si resultat=0, nouvelles sorties et fin de la procedure (LED orange)
	b	$+4		; sinon nouveau test avec 50
	movlw	B'00000101'
	movwf	sorties
	return
	movlw	D'50'
	subwf	b1,0		; soustraction avec 50
	btfss	status,2
	b	$+4
	movlw	B'00010001'	; LED rouge
	movwf	sorties
	return	
	movlw	d'1'
	subwf	b1,0
	btfss	status,2
	return
	movlw	B'00110001'	; buzzer + LED rouge
	movwf	sorties
	movlw	1
	movwf	endtps
	return	

; boucle appelant la temporisation de 100ms 50 fois
; =================================================
boucle1
	movlw	D'50'
	movwf	b2
	call	tempo
	btfss	gpio,3
	b	start
	decfsz	b2,f
	b	$-4
	b	main

; boucle finale appelant la temporisation 8 fois
; ==============================================
bouclefin
	movlw	D'8'
	movwf	b2
	call	tempo
	btfss	gpio,3
	b	start
	decfsz	b2,f
	b	$-4
	b	stop
	
; programme principal
; ===================
main
	call	testtps		; test du temps en cours
	movlw	sorties
	movwf	gpio		; application aux sorties
	decf	b1,f		; decrementation de la variable principale b1 (depart=250)
	btfss	gpio,3		; si bouton poussoir appuy, mise  zero du programme
	b	start
	movf	endtps,f	; temps termin ? (b1=0 ?)
	btfss	status,2
	b	bouclefin
	b	boucle1

	END
